@@ -13,10 +13,11 @@ test_GomAdapter_open_async_close_cb (GObject      *object,
 
    ret = gom_adapter_close_finish(GOM_ADAPTER(object), result, &error);
    g_assert_no_error(error);
-   g_assert(ret);
+   g_assert_true(ret);
 
    *success = TRUE;
    g_main_loop_quit(gMainLoop);
+   g_clear_pointer(&gMainLoop, g_main_loop_unref);
 }
 
 static void
@@ -29,7 +30,7 @@ test_GomAdapter_open_async_open_cb (GObject      *object,
 
    ret = gom_adapter_open_finish(GOM_ADAPTER(object), result, &error);
    g_assert_no_error(error);
-   g_assert(ret);
+   g_assert_true(ret);
 
    gom_adapter_close_async(GOM_ADAPTER(object), test_GomAdapter_open_async_close_cb, user_data);
 }
@@ -43,7 +44,9 @@ test_GomAdapter_open_async (void)
    adapter = gom_adapter_new();
    gom_adapter_open_async(adapter, ":memory:", test_GomAdapter_open_async_open_cb, &success);
    g_main_loop_run(gMainLoop);
-   g_assert(success);
+   g_assert_true(success);
+
+   g_clear_object (&adapter);
 }
 
 gint
